package com.tzld.piaoquan.risk.control.service.rule;

import java.util.*;

/**
 * 规则计算
 *
 * @author supeng
 */
public class RuleEvaluator {
    private final Expression ast;
    private final Set<String> variables = new HashSet<>();

    public RuleEvaluator(String rule) {
        Lexer lexer = new Lexer(rule);
        List<Token> tokens = lexer.tokenize();
        Parser parser = new Parser(tokens);
        this.ast = parser.parse();
        // 收集变量
        collectVariables(ast);
    }

    public boolean evaluate(Map<String, Double> context) {
        return ast.evaluate(context);
    }

    private void collectVariables(Expression expr) {
        if (expr instanceof Comparison) {
            Comparison comp = (Comparison) expr;
            if (comp.getLeftVar() != null) {
                variables.add(comp.getLeftVar());
            }
            if (comp.getRightVar() != null) {
                variables.add(comp.getRightVar());
            }
        } else if (expr instanceof BinaryOp) {
            BinaryOp op = (BinaryOp) expr;
            collectVariables(op.getLeft());
            collectVariables(op.getRight());
        } else if (expr instanceof Parenthesis) {
            Parenthesis paren = (Parenthesis) expr;
            collectVariables(paren.getExpr());
        }
    }

    public Set<String> getVariables() {
        return variables;
    }

    public static void main(String[] args) {
        String rule = "(region_rate_50_>0.4&&region_tgi_50_>100)||(gender_rate_male>0.5||gender_tgi_male>100)";
        RuleEvaluator evaluator = new RuleEvaluator(rule);

        // 获取所有变量名并查询数据库
        Set<String> variables = evaluator.getVariables();
        System.out.println(variables);
        Map<String, Double> context = new HashMap<>();
        context.put("region_rate_50_", 0.4);
        context.put("region_tgi_50_", 0.4);
        context.put("gender_rate_male", 0.4);
        context.put("gender_tgi_male", 0.4);

        // 规则求值
        boolean result = evaluator.evaluate(context);
        System.out.println("Result: " + result);
    }
}
